package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.List;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.procedure.IQueryable;
import com.bleujin.framework.db.procedure.IUserProcedure;
import com.bleujin.framework.db.procedure.UserProcedures;
import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.dimpl.carrier.CarrierConstant;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.value.Value;

public class DatabaseEventProcessor extends EventProcessor implements CarrierConstant{

	
	private DBController dc ;
	private List<ReservedProcedure> procs ;
	
	DatabaseEventProcessor(DBController dc) {
		this.dc = dc ; 
		this.procs = new ArrayList<ReservedProcedure>() ;
	}

	public void process(List<Event> pendingEvent) throws ALRepositoryException {
		handleEvent(pendingEvent);
		dc.execUpdate(getProcedures()) ;
	}

	public void process(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		handleEvent(pendingEvent) ;
		UserProcedures upts = new UserProcedures(thisNode.getName() + " save") ;
		List<Event> aboutEvents = new ArrayList<Event>() ;
		for (ReservedProcedure rp : procs) {
			if (rp.isAboutNode(thisNode)){
				upts.add(rp.getProcedure()) ;
				aboutEvents.add(rp.getAboutEvent()) ;
			} 
		}
		
		for (Event event : aboutEvents) {
			pendingEvent.remove(event) ;
		}
		dc.execUpdate(upts) ;
	}


	private UserProcedures getProcedures() {
		UserProcedures upts = new UserProcedures("session save") ;
		for(ReservedProcedure rp : procs){
			upts.add(rp.getProcedure()) ;
		}
		return upts;
	}

	protected void moveNodeEvent(Event thisEvent, Node node) throws ALRepositoryException {
		IUserProcedure upt = createUserProcedure("node@moveWith(?,?,?,?,?)");
		upt.addParam(0, node.getIdentifier());
		upt.addParam(1, node.getNodeType().getId());
		upt.addParam(2, node.getName());
		upt.addParam(3, node.getRegister().getMyId());
		upt.addParam(4, node.getParent().getIdentifier());
		addProcedure(makeKey(node.getIdentifier(), "node/move"), upt, thisEvent);
	}

	private String makeKey(String uuid, String action) {
		return uuid + "/" + action;
	}
	private void addProcedure(String key, IQueryable upt, Event thisEvent){
		for(ReservedProcedure rp : procs){
			if (key.equals(rp.getKey())) return ;
		}
		Debug.happy("KEY:" + key + " added") ;
		procs.add(new ReservedProcedure(key, upt, thisEvent)) ;
	}
	private void removeProcedure(String key) {
		ReservedProcedure forRemoved = null ;
		for(ReservedProcedure rp : procs){
			if (rp.getKey().startsWith(key)) {
				forRemoved = rp ;
				Debug.happy("KEY:" + key + " removed") ;
				break ;
			}
		}
		if (forRemoved != null) procs.remove(forRemoved) ;
	}


	protected void checkOutEvent(Event thisEvent, NodeVersion node) throws ALRepositoryException {
		int versionNum = (int)node.getVersionHistory().getAllVersions().size() ;
		
		String userId = node.getSession().getUserId() ;
		IUserProcedure upt = createUserProcedure("version@newNodeWith(?,?,?,?)") ;
		upt.addParam(node.getIdentifier()).addParam(versionNum).addParam(userId).addParam(DateUtil.currentGMTToString()) ;
		addProcedure(makeKey(node.getIdentifier(), "version/create"), upt, thisEvent);
		
		for (Property property : node.getProperties()) {
			UserProcedures upts = createUserProcedures("property set") ;
			IUserProcedure pt = createUserProcedure("version@newPropertyWith(?,?,?,?,?)");
			pt.addParam(0, node.getIdentifier());
			pt.addParam(1, versionNum);
			pt.addParam(2, property.getId());
			pt.addParam(3, property.hasArrayValue() ? T : F);
			pt.addParam(4, property.getValue().getValueString());
			upts.add(pt) ;

			if (property.hasArrayValue()){
				IUserProcedure delArray = createUserProcedure("property@removeArrayWith(?,?)") ;
				delArray.addParam(node.getIdentifier()).addParam(property.getId()) ;
				upts.add(delArray) ;
				
				Value[] values = property.getValues() ;
				int idx = 0 ;
				for (Value value : values) {
					Debug.debug(this, value);
					IUserProcedure setArray = createUserProcedure("version@newPropertyArrayWith(?,?,?,?,?)") ;
					setArray.addParam(node.getIdentifier()).addParam(node.getVersionNum()).addParam(property.getId()).addParam(idx++).addParam(value.getValueString()) ;
					upts.add(setArray) ;
				}
			}
			addProcedure(makeKey(node.getIdentifier(), "version/property/add/" + property.getId()), upts, thisEvent);
		}
		removeProcedure(makeKey(node.getIdentifier(), "node/property/change/")) ;
	}

	protected void changePropertyEvent(Event thisEvent, Property property) throws ALRepositoryException {
		UserProcedures upts = getProperyProcedure(property) ;
		addProcedure(makeKey(property.getNode().getIdentifier(), "node/property/change/" + property.getId()), upts, thisEvent);
	}

	protected void createPropertyEvent(Event thisEvent, Property property) throws ALRepositoryException {
		UserProcedures upts = getProperyProcedure(property) ;
		addProcedure(makeKey(property.getNode().getIdentifier(), "node/property/create/" + property.getId()), upts, thisEvent);
	}
	
	private UserProcedures getProperyProcedure(Property property) throws ALRepositoryException {
		UserProcedures upts = createUserProcedures("propertySet") ;
		IUserProcedure upt = createUserProcedure("property@setWith(?,?,?,?)");
		upt.addParam(0, property.getNode().getIdentifier());
		upt.addParam(1, property.getId());
		upt.addParam(2, property.hasArrayValue() ? T : F);
		upt.addParam(3, property.getValue().getValueString());
		upts.add(upt) ;

		if (property.hasArrayValue()){
			IUserProcedure delArray = createUserProcedure("property@removeArrayWith(?,?)") ;
			delArray.addParam(property.getNode().getIdentifier()).addParam(property.getId()) ;
			upts.add(delArray) ;
			
			Value[] values = property.getValues() ;
			int idx = 0 ;
			for (Value value : values) {
				IUserProcedure setArray = createUserProcedure("property@setArrayWith(?,?,?,?)") ;
				setArray.addParam(property.getNode().getIdentifier()).addParam(property.getId()).addParam(idx++).addParam(value.getValueString()) ;
				upts.add(setArray) ;
			}
		}
		return upts ;
	}


	protected void delLinkEvent(Event thisEvent, Link link) {
		// TODO Auto-generated method stub
		
	}

	protected void createNodeEvent(Event thisEvent, INode newNode) throws ALRepositoryException {

		IUserProcedure upt = createUserProcedure("node@createWith(?,?,?,?,?,?,?)");
		upt.addParam(0, newNode.getIdentifier());
		upt.addParam(1, newNode.getNodeType().getId());
		upt.addParam(2, newNode.getName());
		upt.addParam(3, newNode.getExplain());
		upt.addParam(4, newNode.getRegister().getMyId());
		upt.addParam(5, newNode.getParent().getIdentifier());
		upt.addParam(6, DateUtil.currentGMTToString());
		addProcedure(makeKey(newNode.getIdentifier(), "node/create/" + newNode.getName()), upt, thisEvent);

//		if (newNode.getParent().isRoot()){
//			LinkImpl link = LinkImpl.create(newNode.getSession(), newNode.getParent(), newNode, LinkTypeImpl.CHILD, Position.AFTER);
//			Event event = new LinkEvent(link, EventType.LINK_ADDED, thisEvent.getTriggerUser()) ;
//			createLinkEvent(event, link) ;
//		}
	}


	protected void createLinkEvent(Event thisEvent, Link link) {
		IUserProcedure upt = createUserProcedure("link@createWith(?,?,?,?)");
		upt.addParam(0, link.getFromId());
		upt.addParam(1, link.getToId());
		upt.addParam(2, link.getType().toString());
		upt.addParam(3, link.getPosition().toString());
		addProcedure(makeKey(link.getToId(), "link/" + link.getType() + "/" + link.getFromId()), upt, thisEvent);
	}

	protected void delNodeEvent(Event thisEvent, Node node) {
		IUserProcedure upt = createUserProcedure("node@deleteWith(?)");
		upt.addParam(0, node.getIdentifier());
		addProcedure(makeKey(node.getIdentifier(), "node/delete"), upt, thisEvent);
	}
	
	
	private IUserProcedure createUserProcedure(String procSQL){
    	return dc.createUserProcedure(procSQL) ; 
    }

	private UserProcedures createUserProcedures(String procSQL){
    	return dc.createUserProcedures(procSQL) ; 
    }

	@Override
	protected void updateNodeEvent(Event thisEvent, INode node) {
		IUserProcedure upt = createUserProcedure("node@updateWith(?,?,?)");
		upt.addParam(node.getIdentifier()).addParam(node.getName()).addParam(node.getExplain());
		addProcedure(makeKey(node.getIdentifier(), "node/update"), upt, thisEvent);
	}

	
}

class ReservedProcedure{
	private String key ;
	private IQueryable upt ;
	private Event aboutEvent ;
	ReservedProcedure(String key, IQueryable upt, Event aboutEvent) {
		this.key = key ;
		this.upt = upt ;
		this.aboutEvent = aboutEvent ;
	}
	
	String getKey() {
		return key;
	}

	IQueryable getProcedure() {
		return upt;
	}
	
	Event getAboutEvent(){
		return aboutEvent ;
	}

	boolean isAboutNode(Node node){
		return key.startsWith(node.getIdentifier() + "/") ;
	}

}