package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.ExtendTransformer;
import com.bleujin.framework.db.NRows;
import com.bleujin.framework.db.Row;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.procedure.IUserCommand;
import com.bleujin.framework.db.procedure.UserProcedures;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.carrier.CarrierConstant;
import com.bleujin.lore.core.dimpl.carrier.LinkCarrier;
import com.bleujin.lore.core.dimpl.carrier.NodeCarrier;
import com.bleujin.lore.core.dimpl.carrier.PropertyCarrier;
import com.bleujin.lore.core.dimpl.carrier.VersionCarrier;
import com.bleujin.lore.core.dimpl.carrier.WorkspaceCarrier;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.IllegalNodeNameException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.search.indexer.NodeIndexer;
import com.bleujin.lore.core.value.Value;

public class DatabaseLocalStore extends AbstractLocalStore implements CarrierConstant{

	private DBController dc ;
	
	private DatabaseLocalStore(DBController dc, NodeIndexer indexer) {
	    super(indexer);
	    this.dc = dc ;
    }
	
	public final static DatabaseLocalStore create(DBController dc, NodeIndexer indexer) {
		return new DatabaseLocalStore(dc, indexer) ;
	}
	
	public INode createNode(UserSession session, Node parent, NodeType nodetype, NodeName name, String explain) {
		return NodeDBImpl.create(session, parent, nodetype, name, explain) ;
	}
	
    public boolean exist(UserSession session, String path) {
		return getNodeCarrier().exist(path);
    }

    public GenericIterator<Link> getLinks(Node node, Page page) throws ALRepositoryException {
		NRows rows = getLinkCarrier().listBy(node.getIdentifier(), page) ;
		UserSession session = node.getSession();
		List<Link> result = new ArrayList<Link>();
		while(rows.next()){
			String linkTypeName = rows.getString("linkTypeCd");
			Link link = LinkImpl.load(session, node.getIdentifier(), rows.getString("toUUID"), LinkTypeImpl.load(linkTypeName)) ;
			result.add(link) ;
		}
		return new GenericIteratorHelper<Link>(result) ;
    }

	public GenericIterator<Link> getLinks(Node node, LinkType linkType, Page page) throws ALRepositoryException {
		NRows rows = getLinkCarrier().listBy(node.getIdentifier(), linkType, page) ;
		UserSession session = node.getSession();
		List<Link> result = new ArrayList<Link>();
		while(rows.next()){
			String linkTypeName = rows.getString("linkTypeCd");
			Link link = LinkImpl.load(session, node.getIdentifier(), rows.getString("toUUID"), LinkTypeImpl.load(linkTypeName)) ;
			result.add(link) ;
		}
		return new GenericIteratorHelper<Link>(result) ;
	}
	
    public Node getNodeByUUID(UserSession session, String uuid) throws ALRepositoryException {
		Row row = getNodeCarrier().retrieveByUUID(session.getWorkspace().getName(), uuid) ;
		RowToNodeBean rowBean = RowToNodeBean.create(session, row) ;
		return NodeDBImpl.load(session, rowBean) ;
    }

    public Node getNodeByPath(UserSession session, String path) throws ALRepositoryException {
		Row row = getNodeCarrier().retrieveByPath(session.getWorkspace().getName() + path) ;
		RowToNodeBean rowBean = RowToNodeBean.create(session, row) ;
		return NodeDBImpl.load(session, rowBean) ;
    }
	
	public GenericIterator<Node> getChildNodes(Node node, Page page) throws ALRepositoryException {
		NRows rows = getNodeCarrier().listChildBy(node, page) ;
		return new GenericIteratorHelper<Node>(toNodes(node.getSession(), rows));
	}


    public GenericIterator<Node> getNodes(Node node, LinkType type, Page page) throws ALRepositoryException {
		NRows rows = getLinkCarrier().listNodeBy(node.getIdentifier(), type, page) ;
	    return new GenericIteratorHelper<Node>(toNodes(node.getSession(), rows)) ;
    }

	private List<Node> toNodes(UserSession session, NRows nrows) throws IllegalNodeNameException, ALRepositoryException {
		ExtendTransformer trans = new ExtendTransformer(nrows) ;
		Row[] rows = trans.fetchToRowArray() ;
		
		List<Node> result = new ArrayList<Node>();
		for(Row row : rows) {
			result.add(NodeDBImpl.load(session, RowToNodeBean.create(session, row))) ;
		}
		return result;
	}

	public GenericIterator<Node> getNodes(UserSession session, NodeType nodetype, boolean includeSub, Page page) throws ALRepositoryException {
		NRows rows = getNodeCarrier().getNodes(nodetype, session.getWorkspace().getName(), includeSub, page);
	    return new GenericIteratorHelper<Node>(toNodes(session, rows)) ;		
	}


	public GenericIterator<Property> getProperties(Node node) throws ALRepositoryException {
		NRows rows = getPropertyCarrier().getProperties(node.getIdentifier(), node.getVersionNum()) ;
		NodeType nodetype = node.getNodeType() ;

		List<Property> result = new ArrayList<Property>() ;
		while(rows.next()){
			String propId = rows.getString("pdId");
			if (nodetype.containsPropId(propId)){
				Property property = PropertyImpl.load(node, propId, nodetype.getMappedDefinition(propId).getPropertyDefinition().requiredType(), T.equals(rows.getString("hasArray")), rows.getString("value")) ;
				result.add(property) ;
			}
		}
		
		return new GenericIteratorHelper<Property>(result) ;
    }
	
    public VersionHistory getVersionHistory(Node currentNode) throws ALRepositoryException {
		return getVersionCarrier().historyBy(currentNode) ;
    }

    public void copy(Node src, Node destParent) throws ALRepositoryException {
		getNodeCarrier().copy(src.getIdentifier(), UUID.randomUUID().toString(), destParent.getIdentifier()) ;
    }

    public void move(Node src, Node destParent) throws ALRepositoryException {
		getNodeCarrier().move(src.getIdentifier(), destParent.getIdentifier()) ;
    }

	public Value[] getValues(Node node, Property property) throws ALRepositoryException {
		NRows rows = getPropertyCarrier().getValues(node.getIdentifier(), property.getId(), node.getVersionNum()) ;
		List<Value> result = new ArrayList<Value>() ;
		while(rows.next()){
			Value value = PropertyImpl.toValue(node, property.getType(), rows.getString("value")) ;
			result.add(value) ;
		}
		return result.toArray(new Value[0]) ;
	}

    
    public void indexNode(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		//getNodeCarrier().saveNode(thisNode) ;
		
		getIndexer().update(pendingEvent, thisNode) ;
    }
	
	public void indexNode(List<Event> pendingEvent, Collection<Node> thisNodes, Map<String, Link> thisLinks) throws ALRepositoryException {
		// getNodeCarrier().saveSession(thisNodes, thisLinks) ;
		
		getIndexer().update(pendingEvent) ;
	}

	// helper carrier
	private NodeCarrier getNodeCarrier() {
	    return new NodeCarrier(dc);
    }

	private PropertyCarrier getPropertyCarrier() {
	    return new PropertyCarrier(dc);
    }

	private LinkCarrier getLinkCarrier() {
	    return new LinkCarrier(dc);
    }
	
	private WorkspaceCarrier getWorkspaceCarrier(){
		return new WorkspaceCarrier(dc) ;
	}
	
    private VersionCarrier getVersionCarrier() {
		return new VersionCarrier(dc);
	}


	public void clear() {
		getNodeCarrier().clear() ;
	}
	
	public Node getIternalShareNode(UserSession session, String workspaceName, ShareInfo shareInfo) throws ALRepositoryException {
		Row row = getNodeCarrier().retrieveByPath(workspaceName + shareInfo.getNodePath()) ;
		RowToNodeBean rowBean = RowToNodeBean.create(session, row) ;
		return NodeDBImpl.load(session, rowBean) ;
	}

	
	public void confirm(UserSession session) throws ALRepositoryException {
    	IUserCommand npt =  dc.createUserCommand("select * from node_tblc connect by prior uuid = parentId start with parentId = '*'") ;
    	IUserCommand lpt =  dc.createUserCommand("select * from link_tblc x1, node_tblc x2 where x1.toUUID = x2.UUID") ;
    	IUserCommand ppt =  dc.createUserCommand("select * from property_tblc") ;
    	
    	UserProcedures upts = new UserProcedures("confirm").add(npt).add(lpt).add(ppt) ;
    	
    	Rows rows = dc.getRows(upts) ;
    	NRows node = new NRows(rows, new RuntimeException());
    	NRows link = new NRows(rows.getNextRows(), new RuntimeException());
    	NRows property = new NRows(rows.getNextRows().getNextRows(), new RuntimeException());
    	
    	while(node.next()){
    		Debug.debug(node.getString("name") + "[" + node.getString("ntId") + "] " + node.getString("uuid") + "<" + node.getString("versionNum")+ ">");
    		link.beforeFirst() ;
        	while(link.next()){
        		if (node.getString("uuId").equals(link.getString("fromUUID"))) {
        			Debug.debug("    <" + link.getString("linkTypeCd") + "> " + link.getString("name"));
        		}
        	}
        	
        	property.beforeFirst() ;
        	while(property.next()){
        		if (node.getString("uuId").equals(property.getString("uuId"))) {
        			Debug.debug("    [" + property.getString("pdId") + "] : " + property.getString("value"));
        		}
        	}

    	}
        
    	NRows versions = new NRows(dc.getRows("select x2.uuId, x2.name, x1.versionNum from node_version_tblc x1, node_tblc x2 where x1.uuId = x2.uuId"), new RuntimeException());
    	while(versions.next()){
    		Debug.debug(versions.getString("name") + " HAS version " + versions.getString("versionNum"));
    		NRows ps = new NRows(dc.getRows("select * from property_version_tblc where uuid = '" + versions.getString("uuid") + "'"), new RuntimeException());
    		while(ps.next()){
    			Debug.debug(ps.getString("pdId") + " : " + ps.getString("value"));
    		}
    	}

//    	Debug.debug("------ Link");
//    	while(link.next()){
//    		Debug.debug(link.getString("fromUUID") + "->" + link.getString("toUUID"));
//    	}

//    	Debug.debug("------ Property");
//    	while(property.next()){
//    		Debug.debug(property.getString("uuid") + "[" + property.getString("pdId") + "] : " + property.getString("value"));
//    	}
    	Debug.debug("------------------------------------");
    }

	
	
	public void save(List<Event> pendingEvent) throws ALRepositoryException{
		new DatabaseEventProcessor(dc).process(Collections.unmodifiableList(pendingEvent)) ;
	}
	
	public void save(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		new DatabaseEventProcessor(dc).process(pendingEvent, thisNode) ;
	}
	
	public void close() {
		dc._destroy() ;
	}


}

