package com.bleujin.lore.core.dimpl;



import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NoSuchNodeTypeException;
import com.bleujin.lore.core.exception.UnsupportedRepositoryOperationException;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.Property.Type;
import com.bleujin.lore.core.nodetype.InvalidNodeTypeDefinitionException;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeExistsException;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.persitence.NamespaceRegistry;

public interface NodeTypeManager {

    /**
     * Returns an empty <code>NodeTypeTemplate</code> which can then be used to
     * define a node type and passed to
     * <code>NodeTypeManager.registerNodeType</code>.
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code> if this
     * implementation does not support node type registration.
     * @return A <code>NodeTypeTemplate</code>.
     * @throws UnsupportedRepositoryOperationException if this implementation
     *  does not support node type registration.
     * @throws RepositoryException if another error occurs.
     * @since JCR 2.0
     */
    public NewNodeType createNodeTypeTemplate(NodeType superType, String nodeTypeName) throws UnsupportedRepositoryOperationException, ALRepositoryException;
    public NewNodeType createNodeTypeTemplate(NodeType superType, String nodeTypeName, PropertyDefinition[] definitions) throws UnsupportedRepositoryOperationException, ALRepositoryException;

    /**
     * Returns the named node type.
     * <p>
     * Throws a <code>NoSuchNodeTypeException</code> if a node type by that name does not exist.
     * <p>
     * Throws a <code>RepositoryException</code> if another error occurs.
     *
     * @param nodeTypeName the name of an existing node type.
     * @return A <code>NodeType</code> object.
     * @throws NoSuchNodeTypeException if no node type by the given name exists.
     * @throws ALRepositoryException if another error occurs.
     */
    public NodeType getNodeType(String nodeTypeName) throws NoSuchNodeTypeException, ALRepositoryException;

    /**
     * Returns <code>true</code> if a node type with the specified name is
     * registered. Returns <code>false</code> otherwise.
     *
     * @param name a <code>String</code>.
     * @return a <code>boolean</code>
     * @throws ALRepositoryException if an error occurs.
     * @since JCR 2.0
     */
    public boolean hasNodeType(String name) throws ALRepositoryException;

    /**
     * Returns an iterator over all available node types (primary and mixin).
     *
     * @return An <code>NodeTypeIterator</code>.
     *
     * @throws ALRepositoryException if an error occurs.
     */
    public GenericIterator<NodeType> getAllNodeTypes(String prefix) throws ALRepositoryException;

    /**
     * Returns an empty <code>PropertyDefinitionTemplate</code> which can then
     * be used to create a property definition and attached to a
     * <code>NodeTypeTemplate</code>.
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code> if this
     * implementation does not support node type registration.
     *
     * @return A <code>PropertyDefinitionTemplate</code>.
     * @throws UnsupportedRepositoryOperationException if this implementation
     *  does not support node type registration.
     * @throws ALRepositoryException if another error occurs.
     * @since JCR 2.0
     */
    public PropertyDefinition createPropertyDefinition(String propName, Property.Type propertyType) throws UnsupportedRepositoryOperationException, ALRepositoryException;

    /**
     * Registers a new node type or updates an existing node type using the
     * specified definition and returns the resulting <code>NodeType</code>
     * object.
     * <p>
     * Typically, the object passed to this method will be a
     * <code>NodeTypeTemplate</code> (a subclass of
     * <code>NodeTypeDefinition</code>) acquired from
     * <code>NodeTypeManager.createNodeTypeTemplate</code> and then filled-in
     * with definition information.
     * <p>
     * Throws an <code>InvalidNodeTypeDefinitionException</code> if the
     * <code>NodeTypeDefinition</code> is invalid.
     * <p>
     * Throws a <code>NodeTypeExistsException</code> if <code>allowUpdate</code>
     * is <code>false</code> and the <code>NodeTypeDefinition</code> specifies a
     * node type name that is already registered.
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code> if this
     * implementation does not support node type registration.
     *
     * @param ntd an <code>NodeTypeDefinition</code>.
     * @param allowUpdate a boolean
     * @return the registered node type
     * @throws InvalidNodeTypeDefinitionException if the
     *  <code>NodeTypeDefinition</code> is invalid.
     * @throws NodeTypeExistsException if <code>allowUpdate</code> is
     *  <code>false</code> and the <code>NodeTypeDefinition</code> specifies a
     *  node type name that is already registered.
     * @throws UnsupportedRepositoryOperationException if this implementation
     *  does not support node type registration.
     * @throws ALRepositoryException if another error occurs.
     * @since JCR 2.0
     */
    public NodeType register(NewNodeType ntd, boolean allowUpdate) throws InvalidNodeTypeDefinitionException, NodeTypeExistsException, UnsupportedRepositoryOperationException, ALRepositoryException;

    /**
     * Registers or updates the specified array of
     * <code>NodeTypeDefinition</code> objects. This method is used to register
     * or update a set of node types with mutual dependencies. Returns an
     * iterator over the resulting <code>NodeType</code> objects.
     * <p>
     * The effect of the method is "all or nothing"; if an error occurs, no node
     * types are registered or updated.
     * <p>
     * Throws an <code>InvalidNodeTypeDefinitionException</code> if a
     * <code>NodeTypeDefinition</code> within the array is
     * invalid or if the <code>Collection</code> contains an object of a type
     * other than <code>NodeTypeDefinition</code>.
     * <p>
     * Throws a <code>NodeTypeExistsException</code> if <code>allowUpdate</code>
     * is <code>false</code> and a <code>NodeTypeDefinition</code> within the
     * array specifies a node type name that is already
     * registered.
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code> if this
     * implementation does not support node type registration.
     *
     * @param ntds a collection of <code>NodeTypeDefinition</code>s
     * @param allowUpdate a boolean
     * @return the registered node types.
     * @throws InvalidNodeTypeDefinitionException if a
     *  <code>NodeTypeDefinition</code> within the <code>Collection</code> is
     *  invalid or if the <code>Collection</code> contains an object of a type
     *  other than <code>NodeTypeDefinition</code>.
     * @throws NodeTypeExistsException if <code>allowUpdate</code> is
     *  <code>false</code> and a <code>NodeTypeDefinition</code> within the
     *  <code>Collection</code> specifies a node type name that is already
     *  registered.
     * @throws UnsupportedRepositoryOperationException if this implementation
     *  does not support node type registration.
     * @throws ALRepositoryException if another error occurs.
     * @since JCR 2.0
     */
    public GenericIterator<NodeType> register(NewNodeType[] ntds, boolean allowUpdate) throws InvalidNodeTypeDefinitionException, NodeTypeExistsException, UnsupportedRepositoryOperationException, ALRepositoryException;

    /**
     * Unregisters the specified node type.
     * <p>
     * Throws a <code>NoSuchNodeTypeException</code> if no registered node type
     * exists with the specified name.
     *
     * @param name a <code>String</code>.
     * @throws UnsupportedRepositoryOperationException if this implementation
     *  does not support node type registration.
     * @throws NoSuchNodeTypeException if no registered node type exists with
     *  the specified name.
     * @throws ALRepositoryException if another error occurs.
     * @since JCR 2.0
     */
    public void unregister(String name) throws UnsupportedRepositoryOperationException, NoSuchNodeTypeException, ALRepositoryException;

    /**
     * Unregisters the specified set of node types. Used to unregister a set of node types with mutual dependencies.
     * <p>
     * Throws a <code>NoSuchNodeTypeException</code> if one of the names listed is not a registered node type.
     * <p>
     * Throws an <code>UnsupportedRepositoryOperationException</code> if this implementation does not support node type registration.
     *
     * @param names a <code>String</code> array
     * @throws UnsupportedRepositoryOperationException if this implementation does not support node type registration.
     * @throws NoSuchNodeTypeException if one of the names listed is not a registered node type.
     * @throws ALRepositoryException if another error occurs.
     * @since JCR 2.0
     */
    public void unregisterNodeTypes(String[] names) throws UnsupportedRepositoryOperationException, NoSuchNodeTypeException, ALRepositoryException;

    public NodeType register(NodeType superType, String ntId, PropertyDefinition[] definitions) throws InvalidNodeTypeDefinitionException, UnsupportedRepositoryOperationException, ALRepositoryException;
	public NamespaceRegistry getNamespace();

}

class NodeTypeManagerImpl implements NodeTypeManager {

	private GlobalStore gs ;
	public NodeTypeManagerImpl(GlobalStore gs) {
		this.gs = gs ;
	}
	
	public NewNodeType createNodeTypeTemplate(NodeType superType, String nodeTypeName) throws UnsupportedRepositoryOperationException, ALRepositoryException {
		return gs.createNodeType(superType, nodeTypeName, new PropertyDefinition[0]);
	}

	public NewNodeType createNodeTypeTemplate(NodeType superType, String nodeTypeName, PropertyDefinition[] definitions) throws UnsupportedRepositoryOperationException, ALRepositoryException {
		return gs.createNodeType(superType, nodeTypeName, definitions);
	}

	public PropertyDefinition createPropertyDefinition(String propName, Type propertyType) throws UnsupportedRepositoryOperationException, ALRepositoryException {
		return gs.createPropertyDefinition(propName, propertyType);
	}

	public GenericIterator<NodeType> getAllNodeTypes(String prefix) throws ALRepositoryException {
		return gs.getAllNodeTypes(prefix);
	}

	public NamespaceRegistry getNamespace() {
		return gs.getNamespace();
	}

	public NodeType getNodeType(String nodeTypeName) throws NoSuchNodeTypeException, ALRepositoryException {
		return gs.getNodeType(nodeTypeName);
	}

	public boolean hasNodeType(String name) throws ALRepositoryException {
		return gs.includeNodeType(name);
	}

	public NodeType register(NewNodeType newTypes, boolean allowUpdate) throws InvalidNodeTypeDefinitionException, NodeTypeExistsException, UnsupportedRepositoryOperationException,
			ALRepositoryException {
		return register(new NewNodeType[]{newTypes}, allowUpdate).next();
	}

	public GenericIterator<NodeType> register(NewNodeType[] newTypes, boolean allowUpdate) throws InvalidNodeTypeDefinitionException, NodeTypeExistsException,
			UnsupportedRepositoryOperationException, ALRepositoryException {
		return gs.register(newTypes, allowUpdate);
	}

	public NodeType register(NodeType superType, String ntId, PropertyDefinition[] definitions) throws InvalidNodeTypeDefinitionException, UnsupportedRepositoryOperationException,
			ALRepositoryException {
		return register(createNodeTypeTemplate(superType, ntId, definitions), false);
	}

	public void unregister(String name) throws UnsupportedRepositoryOperationException, NoSuchNodeTypeException, ALRepositoryException {
		gs.removeNodeType(name) ;
	}

	public void unregisterNodeTypes(String[] names) throws UnsupportedRepositoryOperationException, NoSuchNodeTypeException, ALRepositoryException {
		throw new UnsupportedRepositoryOperationException() ;
	}
	
}