package com.bleujin.lore.core.dimpl;

import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang.time.DateUtils;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.exception.NodeException;
import com.bleujin.lore.core.mapping.NumberConstraint;
import com.bleujin.lore.core.mapping.PositiveNumberConstraint;
import com.bleujin.lore.core.mapping.RangeByteLengthConstraint;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.valid.PropertyValid;
import com.bleujin.lore.core.value.Value;

public class TestNode extends TestCaseParent {

	private NodeType personType = null ; 
	private String FIRST_NAME = "firstname";
	
	public void setUp() throws Exception {
		super.setUp() ;
		PropertyDefinition firstName = createPropertyDefinition(FIRST_NAME, Property.Type.STRING) ;
		NewNodeType newType = createNewNodeType(objectType, "IPersion", new PropertyDefinition[]{firstName}) ;
		newType.setConstraint(firstName, new RequireConstraint()) ;
		personType = registerNodeType(newType) ;
	}


	public void testRegtDate() throws Exception {
		
		Node node = saveNode(objectType, "abcd") ;
		session.save() ;
		
		String currGMT = DateUtil.currentGMTToString() ;
		Node load = session.getNode("/abcd") ;
		Calendar loadCal = load.getRegister().getCreateDate() ;

		assertEquals(StringUtil.substring(currGMT, 0, 14), StringUtil.substring(DateUtil.calendarToString(loadCal), 0, 14)) ;
	}
	
	public void testPath() throws Exception {
		Node bleujin = saveNode(personType, "bleujin") ;
		Node child = createNodeSave(bleujin, personType, "child") ;

		assertEquals("/bleujin", bleujin.getPath()) ;
		assertEquals(bleujin, bleujin.getNode("/bleujin")) ;
		assertEquals(bleujin, bleujin.getNode("[" + bleujin.getIdentifier() + "]")) ;

		assertEquals("/bleujin/child", child.getPath()) ;
		assertEquals(child, bleujin.getNode("/bleujin/child")) ;
		assertEquals(child, bleujin.getNode("[" + child.getIdentifier() + "]")) ;

		assertEquals(child, bleujin.getNode("child")) ;
		
		assertEquals(session.getRootNode(), bleujin.getNode("")) ;
	}
	
	public void testExplain() throws Exception {
		String message = "Hi Bleujin Explain";
		Node newNode = createNodeSave(personType, "bleujin", message) ;
		assertEquals(message, newNode.getExplain()) ;

		session.save() ;
		
		Node loadNode = session.getLocalStore().getNodeByPath(session, newNode.getPath()) ;
		assertEquals(message, loadNode.getExplain()) ;
	}
	
	
	public void testInsensitiveProperty() throws Exception {
		
		GenericIterator<MappedDefinition> mds = personType.getMappedDefinitions() ;
		assertEquals(1, mds.size()) ;
		
		Node sample = saveNode(personType, "sample") ;
		sample.setProperty(FIRST_NAME, "bleu") ;
		
		assertEquals("bleu", sample.getProperty(FIRST_NAME).getString()) ;
		assertEquals("bleu", sample.getProperty(FIRST_NAME).getString()) ;
		assertEquals("bleu", sample.getProperty(FIRST_NAME).getString()) ;
    }
	
	public void testProperty() throws Exception {
		Node testNode = saveNode(personType, "sample") ;
		testNode.setProperty(FIRST_NAME, Value.stringValue("bleujin")) ;
	    Property property = testNode.getProperty(FIRST_NAME) ;
	    assertNotNull(property) ;
	    assertEquals("bleujin", property.getString()) ;
	}
	
	public void testNotSettingProperty() throws Exception {
		Node testNode = saveNode(personType, "sample") ;
	    assertTrue(testNode.hasProperties()) ;

	    try {
	    	testNode.setProperty("firstName_", Value.stringValue("bleujin")) ;
	    	fail() ;
	    } catch (NoSuchPropertyDefinitionException ignore){
	    }
    }
	
	public void testViewCount() throws Exception {
		Node newNode = saveNode(personType, "sample") ;
		assertEquals(0, newNode.getViewCount()) ;
		session.save() ;
		session.logout() ;
		
		Node loadNode = session.getNode(newNode.getUUIDString()) ;
		
		assertEquals(1, loadNode.getViewCount()) ;
	}
	
	
	public void testValid() throws Exception {
		Node newNode = saveNode(personType, "sample") ;
		
		assertTrue(newNode.hasProperties()) ;
		assertFalse(newNode.getViolateConstraintMessage().size() == 0) ;
		
		PropertyValid valid = new PropertyValid() ;
		assertFalse(valid.isValid(newNode)) ;
    }
	
	public void testProperty2() throws Exception {
	    PropertyDefinition pd_empno = createPropertyDefinition("empNo", Property.Type.STRING);
	    NewNodeType newType = createNewNodeType(personType, "employee", new PropertyDefinition[]{pd_empno}) ;
	    newType.setConstraint(pd_empno, new RequireConstraint()) ;
		NodeType empType = registerNodeType(newType) ;
	    
	    Node empNode = saveNode(empType, "bleujin") ;

	    GenericIterator<Property> iter = empNode.getProperties() ;
	    assertEquals(2, iter.size()) ;
	    
	    empNode.setProperty(FIRST_NAME, "bleujin") ;
	    empNode.setProperty("empNo", "123") ;

	    assertEquals("bleujin", empNode.getProperty(FIRST_NAME).getString()) ;
	    assertEquals(123, empNode.getProperty("empNo").getValue().getLong()) ;

	    session.save() ;
	    
	    assertEquals("bleujin", empNode.getProperty(FIRST_NAME).getString()) ;
	    
	    Node loadEmp = session.getNode(empNode.getUUIDString()) ;

	    
	    assertEquals("bleujin", loadEmp.getProperty(FIRST_NAME).getString()) ;
	    assertEquals(123, loadEmp.getProperty("empNo").getValue().getLong()) ;
    }
	
	public void testDelete() throws Exception {

		Node rootNode = session.getRootNode();
		assertEquals(0, rootNode.getChildNode().size());

		Node oneNode = saveNode(personType, "one") ;
		session.save() ;
		
		GenericIterator<Node> childs = rootNode.getChildNode() ;
		assertEquals(1, rootNode.getChildNode().size());
		assertEquals(oneNode, childs.next()) ;

		oneNode.delete() ;
		
		session.save() ;
		assertEquals(0, rootNode.getChildNode().size());
    }
	
	public void testDeepDelete() throws Exception {
		Node testFirstNode = createTestNode() ;
		Node twoNode = saveNode(personType, "two") ;
		Node two_oneNode = createNodeSave(twoNode, personType, "two-child") ;
		String path = two_oneNode.getPath() ;
		
		session.save() ;
		twoNode.delete() ;
		session.save() ;
		
		assertEquals(0, testFirstNode.getChildNode().size());
		assertEquals(false, session.existByPath(path)) ;
	}
	
	
	public void testParent() throws Exception {
	    Node bleu = saveNode(objectType, "bleu") ;
	    Node jin = createNodeSave(bleu, objectType, "jin") ;
	    session.save() ;
	}
	

	
	public void testParent2() throws Exception {
		String nodeName = "test";
		Node testFirstNode = createNodeSave(session.getRootNode(), objectType, nodeName, "");
		session.save() ;
		
	    PropertyDefinition pd_empno = createPropertyDefinition("empNo", Property.Type.STRING);
	    NewNodeType newType = createNewNodeType(personType, "employee", new PropertyDefinition[]{pd_empno}) ;
	    newType.setConstraint(pd_empno, new RequireConstraint()) ;
		NodeType empType = registerNodeType(newType) ;
	    
	    Node empNode = createNodeSave(testFirstNode, empType, "bleujin") ;
	    empNode.save() ;
	    
	    assertEquals(nodeName, testFirstNode.getName()) ;
	    assertEquals("/" + nodeName, testFirstNode.getPath()) ;
	    assertEquals("bleujin", empNode.getName()) ;
	    assertEquals("/" + nodeName + "/bleujin", empNode.getPath()) ;
	    
	    empNode = createNodeSave(testFirstNode, empType, "bleuhero") ;
	    assertEquals("/" + nodeName + "/bleuhero", empNode.getPath()) ;
	    
	    assertEquals(testFirstNode, empNode.getParent()) ;
	    session.save() ;
    }
	
	
	public void testChild() throws Exception {
	    NodeType boardType = registerNodeType(objectType, "boardName", new PropertyDefinition[]{createPropertyDefinition("boardName", Property.Type.STRING), createPropertyDefinition("createDate", Property.Type.DATE)}) ;

	    PropertyDefinition boardNo = createPropertyDefinition("boardNo", Property.Type.LONG);
		PropertyDefinition subject = createPropertyDefinition("subject", Property.Type.STRING);
		PropertyDefinition regUserId = createPropertyDefinition("regUserId", Property.Type.STRING);
		PropertyDefinition visitCnt = createPropertyDefinition("visitCnt", Property.Type.LONG);
		PropertyDefinition regDate = createPropertyDefinition("regDate", Property.Type.DATE);
		PropertyDefinition content = createPropertyDefinition("content", Property.Type.STRING);
		PropertyDefinition[] articleProperty = new PropertyDefinition[]{
    		boardNo, 
    		subject, 
    		regUserId, 
    		visitCnt, 
    		regDate, 
    		content 
	    } ;
		NewNodeType newArticleType = createNewNodeType(objectType, "article", articleProperty) ;
		newArticleType.setConstraint(boardNo, new NumberConstraint()) ;
		newArticleType.setConstraint(subject, new RangeByteLengthConstraint(1, 200)) ;
		newArticleType.setConstraint(regUserId, new RequireConstraint()) ;
		newArticleType.setConstraint(visitCnt, new PositiveNumberConstraint()) ;
		newArticleType.setConstraint(regDate, new RequireConstraint()) ;
	    NodeType articleType = registerNodeType(newArticleType) ;
	    
	    
	    Node boardNode = saveNode(boardType, "bulletin") ;
	    boardNode.setProperty("boardName", "memoryBook") ;
	    session.save() ;
	    
	    
	    
	    Node article1Node = createNodeSave(boardNode, articleType, "article_1") ;
	    article1Node.save() ;
	    
	    Calendar currDate = Calendar.getInstance() ;
	    currDate.setTime(new Date()) ;
	    article1Node.setProperty(new String[]{"boardNo", "subject", "regUserId", "visitCnt", "regDate", "content"}, 
	    				   new Object[]{1, "article 1", "bleujin", 0, currDate, "Hello bleujin!"}) ;
	    
	    assertEquals(Property.Type.DATE, article1Node.getProperty("regDate").getValue().getPropertyType()) ;

	    assertEquals(1, article1Node.getProperty("boardNo").getValue().getLong()) ;
	    assertEquals("article 1", article1Node.getProperty("subject").getString()) ;
	    assertEquals("bleujin", article1Node.getProperty("regUserId").getString()) ;
	    assertEquals(0, article1Node.getProperty("visitCnt").getValue().getLong()) ;
	    
	    DateUtils.isSameDay(currDate, article1Node.getProperty("regDate").getValue().getDate()) ;
	    DateUtils.isSameInstant(currDate, article1Node.getProperty("regDate").getValue().getDate()) ;
	    assertEquals("Hello bleujin!", article1Node.getProperty("content").getString()) ;


	    session.save() ;
	    
	    GenericIterator<Node> iter = boardNode.getChildNode() ;
	    Debug.debug(this, article1Node.getParent() == boardNode);

	    assertEquals(1, article1Node.getParent().getChildNode().size()) ;
	    assertEquals(1, iter.size()) ;

	    try {
	    	assertEquals(true, session.existByPath(article1Node.getPath())) ; 
	    	
	    	Node duplPathNode = createNodeSave(boardNode, articleType, "article_1") ;
	    	fail() ;
	    } catch(NodeException ignore){
	    	
	    }

	    
	    Node article2Node = createNodeSave(boardNode, articleType, "article_2") ;
	    article2Node.setProperty(new String[]{"boardNo", "subject", "regUserId", "visitCnt", "regDate", "content"}, 
	    				   new Object[]{2, "article 2", "bleujin", 0, currDate, "Hello bleujin!"}) ;
	    
	    session.save() ;
	    assertEquals(2, boardNode.getChildNode().size()) ;
	    
	    boardNode.addMemberNode(article1Node) ;
	    session.save() ;
	    boardNode.addMemberNode(article2Node) ;
	    session.save() ;
	    
	    assertEquals(article2Node, boardNode.getMemberNode(Page.ALL).next()) ;


	    iter = boardNode.getMemberNode(Page.ALL);
		iter.skip(iter.size() - 1);
	    assertEquals(article1Node, iter.next()) ;

	    Node article3Node = createNodeSave(boardNode, articleType, "article_3") ;
	    boardNode.addMemberNode(article3Node) ;

	    
	    iter = boardNode.getMemberNode(Page.ALL);
		iter.skip(iter.size() - 1);
	    assertEquals(article1Node, iter.next()) ;
	}

}
