package com.bleujin.lore.core.dimpl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.ListUtil;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.search.indexer.IndexDirectory;
import com.bleujin.lore.core.search.indexer.NodeIndexer;
import com.bleujin.lore.core.value.Value;

public class TransientLocalStore extends AbstractLocalStore {

	// store : workspace = 1 : 1
	private LocalFileStore nfs ;

	private boolean isModified = false ;
	private Collection<Node> nodes ;
	private Map<String, List<NodeVersion>> versions = new HashMap<String, List<NodeVersion>>();

	private TransientLocalStore(NodeIndexer indexer, LocalFileStore nfs) {
		super(indexer);
		this.nfs = nfs;
	}
	


	public final static TransientLocalStore create(String saveFilePath, String indexDirPath) throws IOException {
		Debug.happy(saveFilePath, TransientLocalStore.class);
		
		IndexDirectory indexDir = IndexDirectory.createIndexDirectory(indexDirPath) ;
		return new TransientLocalStore(new NodeIndexer(indexDir), LocalFileStore.create(saveFilePath));
	}

	public NodeImpl createNode(UserSession session, Node parent, NodeType nodetype, NodeName name, String explain) {
		return NodeMemoryImpl.create(session, parent, nodetype, name, explain);
	}

	public Node getNodeByPath(UserSession session, String path) throws ALRepositoryException {
		return readNodeByPath(session, path);
	}

	private boolean isSameWorkspace(String workspaceName, Node node) {
		return node.getWorkspaceName().equals(workspaceName);
	}

	public boolean exist(UserSession session, String path) throws ALRepositoryException {
		Collection<Node> readNodes = readNodes();
		for (Node node : readNodes) {
			Node thisNode = cloneNode(session, node);
			if (path.equals(session.getWorkspace().getName() + thisNode.getPath())) {
				return true;
			}
		}
		return false;
	}



	private Node cloneNode(UserSession session, Node node) throws ALRepositoryException {
		Node result = node.newClone(session) ;
		
		return result ;
	}

	public GenericIterator<Property> getProperties(Node node) throws ALRepositoryException {
		Node readNode = readNodeByUUID(node.getSession(), node.getIdentifier());
		return readNode.getProperties();
	}

	public Node getNodeByUUID(UserSession session, String uuid) throws ALRepositoryException {
		
		Node node = readNodeByUUID(session, uuid);
		// if (node != null && isSameWorkspace(session.getWorkspace().getName(), node)) {
		if (node != null) {
			((AbstractNode)node).increaseViewCount();
			// if (node instanceof ShareNode) return node ;
			return cloneNode(session, node);
		}
		throw ItemNotFoundException.create("uuid", uuid + " " + isSameWorkspace(session.getWorkspace().getName(), node));
	}

	public GenericIterator<Node> getChildNodes(Node node, Page page) throws ALRepositoryException {
		Collection<Node> readNodes = readNodes() ;
		List<Node> result = new ArrayList<Node>() ;
		for (Node read : readNodes) {
			Node loadNode = cloneNode(node.getSession(), read);
			if (loadNode.getParent().equals(node)) {
				result.add(loadNode) ;
			}
		}
		return new GenericIteratorHelper<Node>(result) ;
	}
	
	public GenericIterator<Node> getNodes(Node node, LinkType type, Page page) throws ALRepositoryException {

		GenericIterator<Link> links = getLinks(node, type, page) ;
		List<Node> result = new ArrayList<Node>() ;
		for(Link link : links){
			if (link.getType().equals(type)){
				Node readNode = getNodeByUUID(node.getSession(), link.getToId()) ;
				// if (node == null) continue ; // deleted
				result.add(readNode) ;
			}
		}
		return new GenericIteratorHelper<Node>(page.subList(result)) ;
	}
	
	public GenericIterator<Node> getNodes(UserSession session, final NodeType nodetype, final boolean includeSub, Page page) throws ALRepositoryException {
		
		Collection<Node> readNodes = readNodes() ;
		List<Node> result = new ArrayList<Node>() ;
		for (Node read : readNodes) {
			Node loadNode = cloneNode(session, read);
			if ( (includeSub && loadNode.getNodeType().isNodeType(nodetype)) || loadNode.getNodeType().getId().equals(nodetype.getId())) {
				result.add(loadNode) ;
			}
		}

		return new GenericIteratorHelper<Node>(result) ;
	}

	public GenericIterator<Link> getLinks(Node node, Page page) throws ALRepositoryException {
		return getLinks(node, null, page) ;
	}

	public GenericIterator<Link> getLinks(Node node, LinkType linktype, Page page) throws ALRepositoryException {
		
		if (node.isRoot()) return new GenericIteratorHelper<Link>(new ArrayList<Link>()) ;
		
		
		NodeImpl readNode = (NodeImpl)readNodeByUUID(node.getSession(), node.getIdentifier());
		List<Link> links = readNode.getIternalLinks() ; // otherwise. must check linktype
		List<Link> result = new ArrayList<Link>() ;
		for(Link link : links){
			if (linktype == null || link.getType().equals(linktype)){
				result.add(LinkImpl.load(node.getSession(), link.getFromId(), link.getToId(), link.getType())) ;
			}
		}
		result = ListUtil.slice(result, page) ;
		return new GenericIteratorHelper<Link>(result);	
	}

	public VersionHistory getVersionHistory(Node node) throws ALRepositoryException {
		List<NodeVersion> history = new ArrayList<NodeVersion>() ;
		Node recentNode = readNodeByUUID(node.getSession(), node.getIdentifier());
		List<NodeVersion> entry = versions.get(node.getIdentifier()) ;
		if (entry != null){
			for (NodeVersion v : entry) {
				history.add(v) ;
			}
		}
		return new TransientVersionHistory(history, recentNode) ;
	}



	public void move(Node src, Node destParent) throws ALRepositoryException {
		throw new IllegalAccessError("exception.node.illegal_access.move");
	}
	
	public Value[] getValues(Node node, Property property) {
		throw new IllegalAccessError("exception.node.illegal_access.getValues");
	}


	public void copy(Node src, Node destParent) throws ALRepositoryException {
		throw new IllegalAccessError("exception.node.illegal_access.copy");
	}

	public void indexNode(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		getIndexer().update(pendingEvent, thisNode);
	}

	public void indexNode(List<Event> pendingEvent, Collection<Node> thisNodes, Map<String, Link> thisLinks) throws ALRepositoryException {
		getIndexer().update(pendingEvent);
	}

	public void clear() {
		try {
			nfs.clear() ;
		} catch (InterruptedException ignore) {
			ignore.printStackTrace();
		} catch (IOException ignore) {
			ignore.printStackTrace();
		}
	}

	public Node getIternalShareNode(UserSession session, String workspaceName, ShareInfo shareInfo) throws ALRepositoryException {
		return readNodeByPath(session, workspaceName, shareInfo.getNodePath()) ;
	}

	public void confirm(UserSession session) throws ALRepositoryException {
		Collection<Node> readNodes = readNodes();
		for (Node node : readNodes) {
			cloneNode(session, node) ;
			Debug.debug(node + "{" + node.getVersionNum()+ "}" + "::" + node.getParent());
			for (Property property : node.getProperties()) {
				Debug.debug("\t" + property);
			}
			for (Link link : node.getLinks(Page.ALL)) {
				Debug.debug("\tLink: " + link.getTo() + "{" + link.getTo().getVersionNum() + "}"+ " - " + link.getType());
			}
		}
		Debug.debug("=========================================");
	}


	private synchronized Collection<Node> readNodes() throws ALRepositoryException {
		try {
			
			if (nodes != null && isModified == false) return nodes ;
			
			this.nodes = nfs.readNode() ;
			isModified = false ;
			return this.nodes ;
			
		} catch (ClassNotFoundException e) {
			throw new ALRepositoryException(e);
		} catch (IOException e) {
			throw new ALRepositoryException(e);
		} catch (InterruptedException e) {
			throw new ALRepositoryException(e);
		}
	}

	private Node readNodeByPath(UserSession session, String path) throws ALRepositoryException {
		Collection<Node> nodes = readNodes() ;
		for (Node node : nodes) {
			Node loadNode = cloneNode(session, node);
			if (isSameWorkspace(session.getWorkspace().getName(), node) && path.equals(loadNode.getPath())) {
				return loadNode ;
			}
		}
		throw ItemNotFoundException.create("path", path);
	}

	private Node readNodeByPath(UserSession session, String workspaceName, String path) throws ALRepositoryException {
		Collection<Node> nodes = readNodes() ;
		for (Node node : nodes) {
			Node loadNode = cloneNode(session, node);
			if (isSameWorkspace(workspaceName, node) && path.equals(loadNode.getPath())) {
				// Debug.debug("DDDDDDDD" + session);
				return loadNode ;
			}
		}
		throw ItemNotFoundException.create("path[workspaceName]", path+"[" + workspaceName + "]");
	}

	
	Node readNodeByUUID(UserSession session, String uuid) throws ALRepositoryException {
		Collection<Node> nodes = readNodes() ;
		for (Node node : nodes) {
			if (uuid.equals(node.getIdentifier())) {
				return cloneNode(session, node);
			} else {
				// Debug.debug(uuid + "  " + node.getIdentifier());
			}
		}
		throw ItemNotFoundException.create("UUID", uuid);
	}
	
	
	public synchronized void save(List<Event> pendingEvent) throws ALRepositoryException {
		new TransientEventProcessor(nfs, versions).process(pendingEvent) ;
		isModified = true ;
		
	}
	public synchronized void save(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		new TransientEventProcessor(nfs, versions).process(pendingEvent, thisNode) ;
		isModified = true ;
	}
	
	public void close() {
		nfs.close() ;
	}




}
