package com.bleujin.lore.core.node;

import com.bleujin.lore.core.common.ItemVisitor;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.AccessDeniedException;
import com.bleujin.lore.core.exception.ItemNotFoundException;

public interface Item {
	
    /**
     * Returns the absolute path to this item.
     * If the path includes items that are same-name sibling nodes properties
     * then those elements in the path will include the appropriate
     * "square bracket" index notation (for example, <code>/a/b[3]/c</code>).
     *
     * @return the path of this <code>Item</code>.
     * @throws ALRepositoryException if an error occurs.
     */
	public String getPath() throws ALRepositoryException;

    /**
     * Returns the parent of this <code>Item</code>.
     *
     * @return The parent of this <code>Item</code>.
     *
     * @throws ItemNotFoundException if there is no parent.  This only happens
     * if this item is the root node of a workspace.
     *
     * @throws AccessDeniedException if the current session does not have
     * sufficient access rights to retrieve the parent of this item.
     *
     * @throws ALRepositoryException if another error occurs.
     */
	public Node getParent() throws ItemNotFoundException, AccessDeniedException, ALRepositoryException;

    /**
     * Returns the name of this <code>Item</code>. The name of an item is the
     * last element in its path, minus any square-bracket index that may exist.
     * If this <code>Item</code> is the root node of the workspace (i.e., if
     * <code>this.getDepth() == 0</code>), an empty string will be returned.
     * <p>
     *
     * @return the (or a) name of this <code>Item</code> or an empty string
     * if this <code>Item</code> is the root node.
     *
     * @throws ALRepositoryException if an error occurs.
     */
	public String getName() throws ALRepositoryException ;
	
    /**
     * Returns the <code>UserSession</code> through which this <code>Item</code>
     * was acquired.
     * Every <code>Item</code> can ultimately be traced back through a series
     * of method calls to the call <code>{@link Session#getRootNode}</code>,
     * <code>{@link Session#getNode}</code>. This method returns that
     * <code>Session</code> object.
     *
     * @return the <code>UserSession</code> through which this <code>Item</code> was
     * acquired.
     * @throws ALRepositoryException if an error occurs.
     */
	public UserSession getSession();
	
    /**
     * Returns the ancestor of the specified depth.
     * An ancestor of depth <i>x</i> is the <code>Item</code> that is <i>x</i>
     * levels down along the path from the root node to <i>this</i>
     * <code>Item</code>.
     * <ul>
     * <li><i>depth</i> = 0 returns the root node.
     * <li><i>depth</i> = 1 returns the child of the root node along the path
     * to <i>this</i> <code>Item</code>.
     * <li><i>depth</i> = 2 returns the grandchild of the root node along the
     * path to <i>this</i> <code>Item</code>.
     * <li>And so on to <i>depth</i> = <i>n</i>, where <i>n</i> is the depth
     * of <i>this</i> <code>Item</code>, which returns <i>this</i>
     * <code>Item</code> itself.
     * </ul>
     * If <i>depth</i> &gt; <i>n</i> is specified then a
     * <code>ItemNotFoundException</code> is thrown.
     * <p>
     *
     * @param depth An integer, 0 &lt;= <i>depth</i> &lt;= <i>n</i> where <i>n</i> is the depth
     * of <i>this</i> <code>Item</code>.
     *
     * @return The ancestor of this
     * <code>Item</code> at the specified <code>depth</code>.
     *
     * @throws ItemNotFoundException if <i>depth</i> &lt; 0 or
     * <i>depth</i> &gt; <i>n</i> where <i>n</i> is the is the depth of
     * this item.
     *
     * @throws AccessDeniedException if the current session does not have
     * sufficient access rights to retrieve the specified node.
     *
     * @throws ALRepositoryException if another error occurs.
     */
    public Item getAncestor(int depth) throws ItemNotFoundException, AccessDeniedException, ALRepositoryException;
    
    /**
     * Returns the depth of this <code>Item</code> in the workspace tree.
     * Returns the depth below the root node of <i>this</i> <code>Item</code>
     * (counting <i>this</i> <code>Item</code> itself).
     * <ul>
     * <li>The root node returns 0.
     * <li>A property or child node of the root node returns 1.
     * <li>A property or child node of a child node of the root returns 2.
     * <li>And so on to <i>this</i> <code>Item</code>.
     * </ul>
     *
     * @return The depth of this <code>Item</code> in the workspace hierarchy.
     * @throws ALRepositoryException if an error occurs.
     */
    public int getDepth() throws ALRepositoryException;


    /**
     * Indicates whether this <code>Item</code> is a <code>Node</code> or a
     * <code>Property</code>.
     * Returns <code>true</code> if this <code>Item</code> is a <code>Node</code>;
     * Returns <code>false</code> if this <code>Item</code> is a <code>Property</code>.
     *
     * @return <code>true</code> if this <code>Item</code> is a
     * <code>Node</code>, <code>false</code> if it is a <code>Property</code>.
     */
    public boolean isNode();

    /**
     * Returns <code>true</code> if this is a new item, meaning that it exists only in transient
     * storage on the <code>Session</code> and has not yet been saved. Within a transaction,
     * <code>isNew</code> on an <code>Item</code> may return <code>false</code> (because the item
     * has been saved) even if that <code>Item</code> is not in persistent storage (because the
     * transaction has not yet been committed).
     * <p>
     * Note that if an item returns <code>true</code> on <code>isNew</code>,
     * then by definition is parent will return <code>true</code> on <code>isModified</code>.
     * <p>
     * Note that in level 1 (that is, read-only) implementations,
     * this method will always return <code>false</code>.
     *
     * @return <code>true</code> if this item is new; <code>false</code> otherwise.
     */
    public boolean isNew();

    /**
     * Returns <code>true</code> if this <code>Item</code> has been saved but has subsequently
     * been modified through the current session and therefore the state of this item as recorded
     * in the session differs from the state of this item as saved. Within a transaction,
     * <code>isModified</code> on an <code>Item</code> may return <code>false</code> (because the
     * <code>Item</code> has been saved since the modification) even if the modification in question
     * is not in persistent storage (because the transaction has not yet been committed).
     * <p>
     * Note that in level 1 (that is, read-only) implementations,
     * this method will always return <code>false</code>.
     *
     * @return <code>true</code> if this item is modified; <code>false</code> otherwise.
     */
    public boolean isModified();

    /**
     * Returns <code>true</code> if this <code>Item</code> object
     * (the Java object instance) represents the same actual workspace item as the
     * object <code>otherItem</code>.
     * <p>
     * Two <code>Item</code> objects represent the same workspace item if all the following
     * are true:
     * <ul>
     *   <li>Both objects were acquired through <code>Session</code> objects that were created
     *       by the same <code>Repository</code> object.</li>
     *   <li>Both objects were acquired through <code>Session</code> objects bound to the same
     *       repository workspace.</li>
     *   <li>The objects are either both <code>Node</code> objects or both <code>Property</code>
     *       objects.</li>
     *   <li>If they are <code>Node</code> objects, they have the same correspondence identifier.
     *       Note that this is the identifier used to determine whether two nodes in different
     *       workspaces correspond but obviously it is also true that any node has the same
     *       correspondence identifier as itself. Hence, this identifier is used here to
     *       determine whether two different Java <code>Node</code> objects actually represent the same
     *       workspace node.</li>
     *   <li>If they are <code>Property</code> objects they have identical names and
     *       <code>isSame</code> is true of their parent nodes.</li>
     * </ul>
     * This method does not compare the <i>states</i> of the two items. For example, if two
     * <code>Item</code> objects representing the same actual workspace item have been
     * retrieved through two different sessions and one has been modified, then this method
     * will still return <code>true</code> when comparing these two objects. Note that if two
     * <code>Item</code> objects representing the same workspace item
     * are retrieved through the <i>same</i> session they will always reflect the
     * same state (see section 5.1.3 <i>Reflecting Item State</i> in the JSR 283 specification
     * document) so comparing state is not an issue.
     *
     * @param otherItem the <code>Item</code> object to be tested for identity with this <code>Item</code>.
     *
     * @return <code>true</code> if this <code>Item</code> object and <code>otherItem</code> represent the same actual repository
     * item; <code>false</code> otherwise.
     *
     * @throws ALRepositoryException if an error occurs.
     */
    public boolean isSame(Item otherItem) throws ALRepositoryException;

    /**
     * Accepts an <code>ItemVisitor</code>.
     * Calls the appropriate <code>ItemVisitor</code>
     * <code>visit</code> method of the according to whether <i>this</i>
     * <code>Item</code> is a <code>Node</code> or a <code>Property</code>.
     *
     * @param visitor The ItemVisitor to be accepted.
     *
     * @throws RepositoryException if an error occurs.
     */
    public void accept(ItemVisitor visitor) throws ALRepositoryException;
    
    // save(), refresh(boolean keepChanges), remove()
}
