package com.bleujin.lore.core.search.indexer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.Term;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.dimpl.EventProcessor;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.search.ISearcher;
import com.bleujin.lore.core.value.Value;

public class IndexEventProcess extends EventProcessor implements ISearcher{

	
	private IndexerHelper helper ;
	private Map<String, WeightEvent> events = new HashMap<String, WeightEvent>() ;
	public IndexEventProcess(IndexerHelper helper) {
		this.helper = helper ;
	}
	

	@Override
	public void process(List<Event> pendingEvent) throws ALRepositoryException {
		handleEvent(pendingEvent) ;
		
		updateIndex() ;
		events.clear() ;
	}

	@Override
	public void process(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		// handleEvent(pendingEvent) ;
		
		put(WeightEvent.updateEvent(thisNode)) ;
		updateIndex() ;
		events.clear() ;
	}	
	
	@Override
	protected void changePropertyEvent(Event event, Property property) throws ALRepositoryException {
		put(WeightEvent.updateEvent(property.getNode())) ;
	}

	@Override
	protected void checkOutEvent(Event event, NodeVersion node) throws ALRepositoryException {
		put(WeightEvent.updateEvent(node)) ;
	}

	@Override
	protected void createNodeEvent(Event event, INode node) throws ALRepositoryException {
		put(WeightEvent.createEvent(node)) ;
	}

	@Override
	protected void delNodeEvent(Event event, Node node) throws ALRepositoryException {
		put(WeightEvent.deleteEvent(node)) ;
	}

	@Override
	protected void moveNodeEvent(Event event, Node node) throws ALRepositoryException {
		put(WeightEvent.updateEvent(node)) ;
	}


	@Override
	protected void updateNodeEvent(Event event, INode node) throws ALRepositoryException {
		put(WeightEvent.updateEvent(node)) ;
	}


	private void put(WeightEvent wevent){
		WeightEvent savedEvent = events.get(wevent.getNodeIdentifier()) ;
		if (savedEvent == null || savedEvent.isLess(wevent)){
			events.put(wevent.getNodeIdentifier(), wevent) ;
		}
	}
	
	
	private void updateIndex() throws ALRepositoryException {
		try {
			helper.updateNode(makeTermForDelete(), makeDocument()) ;
        } catch (IOException e) {
        	throw new ALRepositoryException(e) ;
        } catch (InterruptedException e) {
        	throw new ALRepositoryException(e) ;
        }
	}


	
	private List<Term> makeTermForDelete() {
		List<Term> result = new ArrayList<Term>() ;
		for (WeightEvent event : events.values()) {
			INode node = event.getNode() ;
			if (node.getMode().isCreated()) continue ;
			result.add(new Term(DOC_ID, node.getIdentifier()));
		}
		return result ;
    }

	private List<Document> makeDocument() throws ALRepositoryException {
		// Document[] docs = new Document[(int) nodes.getSize()];
		List<Document> result = new ArrayList<Document>() ;
		int i = 0;
		for (WeightEvent event : events.values()) {
			INode node = event.getNode() ;
			Debug.happy(this.getClass(), node + " " + node.getMode() + events.size());
			if (node.getMode().isDeleted()) continue ;

			MyDocument doc = new MyDocument();
			doc.addKeyword(DOC_ID, node.getIdentifier());
			doc.addKeyword(Node.WORKSPACE, node.getWorkspaceName());
			doc.addKeyword(Node.PATH, node.getPath());
			doc.addKeyword(Node.UUID, node.getIdentifier());
			// doc.addKeyword(Node.STATUS, node.getStatus());
			doc.addKeyword(Node.NODETYPE_ID, node.getNodeType().getName());
			doc.addKeyword("register", node.getRegister().getMyId());
			doc.addKeyword("regtime", DateUtil.calendarToString(node.getRegister().getCreateDate()));

			doc.addText(ALL_FIELD, node.toXML().toString());
			doc.addText(Node.NAME, node.getName());

			if (node.isRoot() == false) {
				doc.addText(Node.PARENT, node.getParent().getPath());
			}

			saveProperty(node.getProperties(), doc);

			result.add(doc.getDocument()) ;
		}
		return result;
	}
	
	private void saveProperty(GenericIterator<Property> properties, MyDocument doc) throws ValueFormatException {
		while (properties.hasNext()) {
			Property property = properties.next();
			String propId = property.getId().getString();
			Value value = property.getValue() ;

			if (property.getValue().isEmpty())
				continue;
			if (property.getType().equals(property.getType().equals(Property.Type.LONG))) {
				doc.addKeyword(propId, value.getLong());
			} else {
				doc.addKeyword(propId, value.getString());
			} 
		}
	}


	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

	@Override
	protected void createLinkEvent(Event event, Link link) throws ALRepositoryException {
		;
	}

	@Override
	protected void createPropertyEvent(Event event, Property property) throws ALRepositoryException {
		;
	}

	@Override
	protected void delLinkEvent(Event event, Link link) throws ALRepositoryException {
		;
	}


}

class WeightEvent{
	
	private int weight ;
	private INode target ;
	
	final static int CREATE_WEIGHT = 1 ;
	final static int UPDATE_WEIGHT = 2 ;
	final static int DELETE_WEIGHT = 3 ;
	
	private WeightEvent(int weight, INode node) {
		this.weight = weight ;
		this.target = node ;
	}
	
	final static WeightEvent createEvent(INode node) {
		return new WeightEvent(CREATE_WEIGHT, node) ;
	}
	final static WeightEvent updateEvent(INode node) {
		return new WeightEvent(UPDATE_WEIGHT, node) ;
	}
	final static WeightEvent deleteEvent(INode node) {
		return new WeightEvent(DELETE_WEIGHT, node) ;
	}
	
	public INode getNode(){
		return target ;
	}
	
	public String getNodeIdentifier(){
		return target.getIdentifier() ;
	}
	
	public boolean isLess(WeightEvent other){
		return this.weight <= other.weight ; 
	}
	
	public String toString(){
		return "event[" + weight +  "] : " + target ;
	}
	
}
