package com.bleujin.lore.core.search.indexer;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.IndexReader.FieldOption;
import org.apache.lucene.store.Directory;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.CaseInsensitiveHashMap;
import com.bleujin.lore.core.common.lock.DefaultReadsWriteLock;
import com.bleujin.lore.core.common.lock.ReadsWriteLock;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.search.ALAnalyzer;
import com.bleujin.lore.core.search.ISearcher;
import com.bleujin.lore.core.search.limo.IndexFile;
import com.bleujin.lore.core.search.limo.LimoUtils;

public class IndexerHelper implements ISearcher {
	public final static boolean INCLUDE_FILE = true;
	public final static boolean INCLUDE_NOT_FILE = false;
	
	private final Directory dir;
	private final Analyzer analyzer;
	private final ReadsWriteLock lock;
	private final String indexRoot;

	IndexerHelper(IndexDirectory dir) {
		this(dir, new DefaultReadsWriteLock());
	}

	IndexerHelper(IndexDirectory dir, ReadsWriteLock lock) {
		this.dir = dir;
		this.analyzer = ALAnalyzer.CJK;
		this.lock = lock;
		this.indexRoot = dir.getIndexPath();
	}

	Directory getDirectory(){
		return this.dir ;
	}
	
	Map<String, Object> getIndexInfo() throws IOException, InterruptedException {
		CaseInsensitiveHashMap<String, Object> map = new CaseInsensitiveHashMap<String, Object>();
		lock.readLock();

		IndexReader reader = null;
		try {
			reader = getReader();
			map.put("numDoc", new Integer(reader.numDocs()));
			String[] field = (String[]) reader.getFieldNames(FieldOption.ALL).toArray(new String[0]);
			map.put("fieldNum", new Integer(field.length));
			map.put("indexedfieldNum", new Integer(reader.getFieldNames(FieldOption.INDEXED).size()));
			map.put("indexPath", indexRoot);
			map.put("fieldName", StringUtils.join(field, ", "));

			List indexFiles = LimoUtils.getIndexFiles(indexRoot);
			if (indexFiles.size() > 0) {
				IndexFile indexFile = (IndexFile) indexFiles.get(0);
				map.put("lastModified", indexFile.getLastModified());
			}
		} finally {
			closeReader(reader);
			lock.readUnLock();
		}
		return map;
	}

	void removeNode(List<Term> terms) throws IOException, InterruptedException {
		lock.writeLock();
		IndexReader reader = null;
		try {
			reader = getReader();
			for(Term term : terms){
				reader.deleteDocuments(term) ;
			}
		} finally {
			closeReader(reader);
			lock.writeUnLock();
		}
	}

	// Writer...
	void addNode(List<Document> docs) throws IOException, InterruptedException, ALRepositoryException {

		lock.writeLock();
		IndexWriter writer = null;
		try {
			writer = getWriter();
			for (Document doc : docs) {
				writer.addDocument(doc);
			}

		} finally {
			closeWriter(writer);
			lock.writeUnLock();
		}
	}
	
	public void updateNode(List<Term> terms, List<Document> docs) throws IOException, InterruptedException, ALRepositoryException {
		lock.writeLock();
		IndexWriter writer = null;
		try {
			writer = getWriter();
			for(Term term : terms){
				writer.deleteDocuments(term) ;
			}
			
			for (Document doc : docs) {
				writer.addDocument(doc);
			}
		
		} finally {
			closeWriter(writer);
			lock.writeUnLock();
		}
	}


	private IndexReader getReader() throws IOException {
		if (!IndexReader.indexExists(dir)) {
			IndexWriter writer = getWriter();
			closeWriter(writer) ;
		}

		return IndexReader.open(dir);
	}

	private IndexWriter getWriter() throws IOException {
		IndexWriter writer = new IndexWriter(dir, analyzer, IndexWriter.MaxFieldLength.LIMITED);
		// writer.setUseCompoundFile(true);
		return writer;
	}

	private void closeReader(IndexReader reader) {
		if (reader == null)
			return;
		try {
			reader.close();
			reader = null;
		} catch (IOException ignore) {
			Debug.warning("at closing reader..", ignore.getMessage());
		}
	}

	private void closeWriter(IndexWriter writer) {
		if (writer == null)
			return;
		try {
			writer.close();
			writer = null;
		} catch (IOException ignore) {
			Debug.warning("at closing writer..", ignore.getMessage());
		}
	}

}
