package com.bleujin.lore.core.search.indexer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import junit.framework.Assert;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.Term;
import org.apache.lucene.store.Directory;

import com.bleujin.framework.logging.LogBroker;
import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ALRepositoryRuntimeException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.search.ISearcher;
import com.bleujin.lore.core.value.Value;

public class NodeIndexer implements ISearcher{
	
	private Logger log = null;
	private IndexerHelper helper = null;

	public NodeIndexer(IndexDirectory dir) {
		Assert.assertNotNull("dir is null...", dir);
		this.log = LogBroker.getLogger(NodeIndexer.class);
		this.helper = new IndexerHelper(dir);
		getIndexInfo() ; // if not exist, null pointer exception occured.. -_-
	}

	public Map getIndexInfo() {
		try {
	        return helper.getIndexInfo();
        } catch (IOException e) {
        	throw new ALRepositoryRuntimeException(e) ;
        } catch (InterruptedException e) {
        	throw new ALRepositoryRuntimeException(e) ;
        }
	}

	// create new....
	void add(Collection<Node> nodes) throws ALRepositoryException {
		try {
	        helper.addNode(makeDocument(nodes));
        } catch (IOException e) {
        	throw new ALRepositoryException(e) ;
        } catch (InterruptedException e) {
        	throw new ALRepositoryException(e) ;
        }
	}

	void update(Node node) throws ALRepositoryException {
		List<Node> nodes = new ArrayList<Node>() ;
		nodes.add(node) ;
		update(nodes) ;
	}

	void update(Collection<Node> nodes) throws ALRepositoryException {
		try {
	        helper.removeNode(makeTermForDelete(nodes));
	        helper.addNode(makeDocument(nodes));
        } catch (IOException e) {
        	throw new ALRepositoryException(e) ;
        } catch (InterruptedException e) {
        	throw new ALRepositoryException(e) ;
        }
	}
	
	
	
	
	void update(Collection<Node> nodes, CancelLog cancel) throws ALRepositoryException {
		try {
			helper.removeNode(makeTermForDelete(nodes));
			helper.addNode(makeDocument(nodes));
		} catch (IOException ex) {
			cancel.cancel();
        	throw new ALRepositoryException(ex) ;
        } catch (InterruptedException e) {
        	throw new ALRepositoryException(e) ;
		}
	}


	void remove(Collection<Node> nodes) throws ALRepositoryException {
		try {
	        helper.removeNode(makeTermForDelete(nodes));
        } catch (IOException e) {
        	throw new ALRepositoryException(e) ;
        } catch (InterruptedException e) {
        	throw new ALRepositoryException(e) ;
        }
	}
	
	
	public void update(List<Event> pendingEvent) throws ALRepositoryException {
		new IndexEventProcess(helper).process(pendingEvent) ;
	}

	public void update(List<Event> pendingEvent, Node node) throws ALRepositoryException {
		new IndexEventProcess(helper).process(pendingEvent, node) ;
	}

	
	public Directory getDirectory(){
		return helper.getDirectory() ;
	}

	
	private List<Term> makeTermForDelete(Collection<Node> nodes) {
		List<Term> result = new ArrayList<Term>() ;
		for (Node node : nodes) {
			if (node.getMode().isCreated()) continue ;
			result.add(new Term(DOC_ID, node.getIdentifier()));
		}
		return result ;
    }

	private List<Document> makeDocument(Collection<Node> nodes) throws ALRepositoryException {
		// Document[] docs = new Document[(int) nodes.getSize()];
		List<Document> result = new ArrayList<Document>() ;
		int i = 0;
		for (Node node : nodes) {
			Debug.debug(this, node + " " + node.getMode());
			if (node.getMode().isDeleted()) continue ;

			MyDocument doc = new MyDocument();
			doc.addKeyword(DOC_ID, node.getIdentifier());
			doc.addKeyword(Node.WORKSPACE, node.getWorkspaceName());
			doc.addKeyword(Node.PATH, node.getPath());
			doc.addKeyword(Node.UUID, node.getIdentifier());
			doc.addKeyword(Node.STATUS, node.getStatus());
			doc.addKeyword(Node.NODETYPE_ID, node.getNodeType().getName());
			doc.addKeyword("register", node.getRegister().getMyId());
			doc.addKeyword("regtime", DateUtil.calendarToString(node.getRegister().getCreateDate()));

			doc.addText(ALL_FIELD, node.toXML().toString());
			doc.addText(Node.NAME, node.getName());

			if (node.isRoot() == false) {
				doc.addText(Node.PARENT, node.getParent().getPath());
			}

			saveProperty(node.getProperties(), doc);

			result.add(doc.getDocument()) ;
		}
		return result;
	}
	
	private void saveProperty(GenericIterator<Property> properties, MyDocument doc) throws ValueFormatException {
		while (properties.hasNext()) {
			Property property = properties.next();
			String propId = property.getId().getString();
			Value value = property.getValue() ;

			if (property.getValue().isEmpty())
				continue;
			if (property.getType().equals(property.getType().equals(Property.Type.LONG))) {
				doc.addKeyword(propId, value.getLong());
			} else {
				doc.addKeyword(propId, value.getString());
			} 
		}
	}

}
