package com.bleujin.lore.core.search.searcher;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.Term;
import org.apache.lucene.misc.ChainedFilter;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Filter;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.QueryFilter;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.SortField;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.WildcardQuery;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.store.Directory;

import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.SearchException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.search.ALAnalyzer;
import com.bleujin.lore.core.search.ISearcher;

public class NodeSearcher implements ISearcher {

	private UserSession session ;
	private SearcherHelper helper;
	private ALQueryParser parser;
	private Node startNode ;

	public NodeSearcher(UserSession session, Node startNode, Directory dir) {
		this(session, startNode, dir, ALAnalyzer.CJK);
	}

	public NodeSearcher(UserSession session, Directory dir) {
		this(session, session.getRootNode(), dir, ALAnalyzer.CJK);
	}

	private NodeSearcher(UserSession session, Node startNode, Directory dir, Analyzer analyzer) {
		this.session = session ;
		this.startNode = startNode ;
        this.helper = new SearcherHelper(dir);
        this.parser = new ALQueryParser(analyzer);
	}

	
	private int DEFAULT_MAX = 1000;
	public SearchResult search(String searchCondition) throws ALRepositoryException {
		return search(searchCondition, "", DEFAULT_MAX) ;
	}
	
	public SearchResult search(String searchCondition, int maxCount) throws ALRepositoryException {
		return search(searchCondition, "", maxCount) ;
	}
	
	private List<Filter> filters = new ArrayList<Filter>() ;
	public void addFilter(String about, String term){
		filters.add(new QueryFilter(new TermQuery(new Term(about, term)))) ;
	}
	
	private ChainedFilter getChainedFilter(){
		if (filters.size() == 0) return null ;
		ChainedFilter chainFilter = new ChainedFilter(filters.toArray(new Filter[0]), ChainedFilter.AND) ;
		return chainFilter ;
	}

	private SearchResult search(String searchCondition, String orderby, int maxCount) throws ALRepositoryException{
		try {
			Query query = parser.parse(searchCondition);
        	//TermQuery workspaceQuery = new TermQuery(new Term(Node.WORKSPACE, startNode.getBean().getWorkspaceName())) ;
        	//Filter workspaceFilter = new QueryFilter(workspaceQuery) ;
			
	        if (startNode.isRoot()== false){
	        	WildcardQuery pathQuery = new WildcardQuery(new Term(Node.PATH, startNode.getPath() + "/*"));
	        	Filter pathFilter = new QueryFilter(pathQuery) ;
	        	filters.add(pathFilter) ;
	        } 
	        
	        return toResult(helper.search(query, getChainedFilter(), getSort(orderby), maxCount));
        } catch (ParseException e) {
	        throw new SearchException(e.getMessage(), e.getCause()) ;
        }
	}

	public SearchResult searchField(String field, String termValue, Page page) throws SearchException {
		Query query = getQuery(field, termValue);
		return toResult(helper.search(query, page));
	}

	public SearchResult searchField(String[] fields, String[] values, boolean isAnd, Page page) throws SearchException {
		BooleanQuery query = new BooleanQuery();
		for (int i = 0, last = values.length; i < last; i++) {
			query.add(getQuery(fields[i], values[i]), Occur.MUST) ;
//			query.add(getQuery(fields[i], values[i]), isAnd, false);
		}

		return toResult(helper.search(query, page));
	}

	private SearchResult toResult(List<Document> result) {
		return new SearchResult(session, startNode, result);
	}


	private boolean isKeywordType(String field) {
		return Node.PATH.equalsIgnoreCase(field) || Node.UUID.equalsIgnoreCase(field);
	}

	private Sort getSort(String orderby) {
		if (StringUtils.isBlank(orderby)) { // no sort option..
			return Sort.RELEVANCE;
			// return new Sort(new SortField[]{SortField.FIELD_SCORE});
		} else {
			String[] orders = StringUtils.split(orderby, ',');
			SortField[] sorts = new SortField[orders.length];
			for (int i = 0, last = orders.length; i < last; i++) {
				String order = orders[i].trim();
				String property = StringUtils.substringBefore(order, " ");
				boolean isASC = "asc".equalsIgnoreCase(StringUtils.defaultString(StringUtils.substringAfterLast(order, " "), "asc")) ? false : true;

				sorts[i] = new SortField(property, SortField.STRING, isASC);
			}

			return new Sort(sorts);
		}
	}

	private Query getQuery(String field, String termValue) throws SearchException {
		try {
	        Query query;
	        if (isKeywordType(field)) {
	        	query = new QueryParser(field, ALAnalyzer.KEYWORD).parse(termValue) ;
	        } else {
	        	query = parser.parse(field + ":" + termValue);
	        }
	        return query;
        } catch (ParseException e) {
	        throw new SearchException(e.getMessage(), e.getCause()) ;
        }
	}
}
