package com.bleujin.lore.core.value;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.db.procedure.IStringObject;
import com.bleujin.framework.util.StringUtil;
import com.bleujin.framework.valid.IValueObject;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.BinaryPropertyValue;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public abstract class Value implements IValueObject, IStringObject, Serializable {
 
	private static final long serialVersionUID = -2614671541056314412L;

	public static final Value EMPTY = new UndefinedValue("");

	private final static int NUMBER_MAX_LENGTH = 24;
	private final static int NUMBER_PREFIX = 0;

	// STRING, BINARY, LONG, DOUBLE, DATE, BOOLEAN, NAME, PATH, REFERENCE, TEXT,
	// INTEGER, VIRTUAL_TEXT, UNDEFINED;
	public final static Value stringValue(String value) {
		return new StringValue(value);
	}

	public static Value binaryValue(BinaryPropertyValue value) {
		return new BinaryValue(value);
	}

	public final static Value longValue(long value) {
		return new LongValue(value);
	}

	public final static Value doubleValue(double value) {
		return new DoubleValue(value);
	}

	public final static Value dateValue(Calendar value) {
		return new DateValue(value);
	}

	public final static Value booleanValue(boolean value) {
		return new BooleanValue(value);
	}

	public final static Value nameValue(String value) {
		return new NameValue(value);
	}

	public final static Value pathValue(String value) {
		return new PathValue(value);
	}

	public final static Value referenceValue(Node value) {
		return new ReferenceValue(value);
	}

	public final static Value textValue(String value) {
		return new TextValue(value);
	}

	public final static Value decimalValue(BigDecimal value) {
		return new DecimalValue(value);
	}

	public final static Value virtualTextValue(String value) {
		return new VirtualTextValue(value);
	}
	
	public final static Value arrayValue(Property.Type propertyType, Serializable... values) {
		ArrayValue avalue = new ArrayValue(propertyType, values) ;
		return avalue ;
	}

	public final static Value newValue(Property.Type propertyType, Object value) {
		if (propertyType == Property.Type.DATE) {
			return (dateValue((Calendar) value));
		} else if (propertyType == Property.Type.BINARY) {
			return binaryValue((BinaryPropertyValue) value);
		} else if (propertyType == Property.Type.LONG) {
			return longValue(value == null ? NUMBER_PREFIX : (Long.valueOf(value.toString())).longValue());
		} else if (propertyType == Property.Type.DOUBLE) {
			return doubleValue(value == null ? NUMBER_PREFIX : (Double.valueOf(value.toString())).doubleValue());
		} else if (propertyType == Property.Type.BOOLEAN) {
			return booleanValue(value == null ? false : new Boolean(value.toString()).booleanValue());
		} else if (propertyType == Property.Type.NAME) {
			return nameValue(value == null ? null : value.toString());
		} else if (propertyType == Property.Type.PATH) {
			return pathValue((String) value);
		} else if (propertyType == Property.Type.REFERENCE) {
			return referenceValue((Node) value);
		} else if (propertyType == Property.Type.STRING) { // String or StringBuffer
			return stringValue(value == null ? null : value.toString());
		} else if (propertyType == Property.Type.TEXT) { // String or StringBuffer
			return textValue(value == null ? null : value.toString());
		} else if (propertyType == Property.Type.DECIMAL) {
			return decimalValue(value == null ? new BigDecimal(NUMBER_PREFIX) : new BigDecimal(value.toString()));
		} else if (propertyType == Property.Type.VIRTUAL_TEXT) {
			return virtualTextValue(value == null ? null : value.toString());
		} else {
			// throw new NotMatchePropertyTypeException("requireType:" + propertyType + ", current:" + value.getClass()) ;
			return undefinedValue((String) value);
		}
	}

	public final static Value undefinedValue(String value) {
		return new UndefinedValue(value);
	}

	public static boolean isMatchedType(Property.Type type, Object value) {
		if (type.isBinary())
			return value instanceof BinaryPropertyValue;
		else if (type.isLong()) {
			if ( (value instanceof Long) || (value instanceof Integer)) return true;
			if ( (value instanceof String) && StringUtil.isNotBlank((String)value) && StringUtil.isNumeric((String)value)) return true ;
			return false ;
		} else if (type.isDouble())
			return (value instanceof Double);
		else if (type.isDate())
			return (value instanceof Calendar);
		else if (type.isPath())
			return (value instanceof String);
		else if (type.isName())
			return (value instanceof IDString);
		else if (type.isText())
			return ((value instanceof String) || (value instanceof StringBuffer));
		// case PropertyType.PATH:
		// return getString();
		// case PropertyType.REFERENCE:
		// return getString();
		else if (type.isDecimal())
			return value instanceof BigDecimal;
		else
			return true;
	}

	public boolean equals(Object that) {
		if (!(that instanceof Value))
			return false;

		Value thatValue = (Value) that;
		return (getPropertyType() == thatValue.getPropertyType()) && getString() != null && getString().equals(thatValue.getString());
	}

	public int hashCode() {
		if (getString() == null) {
			return getPropertyType().hashCode();
		} else {
			return getPropertyType().hashCode() + getString().hashCode();
		}
	}

	public boolean isEmpty() {
		return StringUtil.isEmpty(getString());
	}

	public boolean isBlank() {
		return StringUtil.isBlank(getString());
	}

	public abstract String getString();

	public abstract String getValueString();

	public abstract Object getObject(Map param, Node node) throws ALRepositoryException;

	public abstract InputStream getStream() throws IOException, ValueFormatException;

	public abstract long getLong() throws ValueFormatException;

	public abstract BigDecimal getDecimal() throws ValueFormatException;

	public abstract double getDouble() throws ValueFormatException;

	public abstract Calendar getDate() throws ValueFormatException;

	public abstract boolean getBoolean() throws ValueFormatException;
	
	public long getLength() {
		try {
			return getString().getBytes("UTF-8").length;
		} catch (UnsupportedEncodingException ignore) {
			ignore.printStackTrace();
			return -1 ;
		}
	}

	public abstract Property.Type getPropertyType();

	public abstract Value newClone() ;
	
	public String toString(){
		return getString() ;
	}

}